@@ -20,11 +20,6 @@ try:
 except ImportError:
     print('CUDA extension for flash-cosine-sim-attention was not compiled correctly - please run `pip install flash-cosine-sim-attention --force-reinstall --no-cache-dir`')
 
-# constants
-
-ALLOWED_DIMS = (32, 64, 128)
-ALLOWED_HALF_DIMS = (32, 64, 128)
-
 # helper functions
 
 def exists(val):
@@ -109,8 +104,7 @@ def plain_cosine_sim_attention(
     scale = 10,
     causal = False,
     l2norm_qk = True,
-    attn_bias_batch_dim = False,
-    map_qk_fn: callable = None
+    attn_bias_batch_dim = False
 
 ):
     assert not (causal and exists(mask)), 'mask should not be supplied if causality is needed'
@@ -119,9 +113,6 @@ def plain_cosine_sim_attention(
     if l2norm_qk:
         q, k = l2norm_tensors(q, k)
 
-    if exists(map_qk_fn):
-        q, k = map_qk_fn(q, k)
-
     kv_einsum_eq = 'b j d' if single_head_kv else 'b h j d'
     sim = einsum(f'b h i d, {kv_einsum_eq} -> b h i j', q, k)
     sim = sim * scale
@@ -155,47 +146,26 @@ class FlashCosineSimAttention(Function):
         causal,
         attn_bias_batch_dim
     ):
-        batch, heads, seq, dim_qk, dim, device, dtype = *q.shape, v.shape[-1], q.device, q.dtype
-        assert dim_qk == dim, 'query / key head dimension must be equal to value head dimension for now'
-
-        single_head_kv = k.ndim == 3
-        is_half = dtype == torch.float16
-
-        assert is_half or (dim in ALLOWED_DIMS), f'query key dimension must be one of {ALLOWED_DIMS}'
-        assert (not is_half) or (dim in ALLOWED_HALF_DIMS), f'half dimensions must be one of {ALLOWED_HALF_DIMS}'
-
-        mask = default(mask, lambda: torch.empty(q.shape[0], 0, device = q.device, dtype = torch.bool))
-
-        attn_bias = default(attn_bias, torch.empty(1, 0, 0, device = q.device, dtype = dtype))
-
-        should_backwards = any([*map(lambda t: t.requires_grad, (q, k, v, attn_bias))])
-
-        if single_head_kv:
-            k, v = map(lambda t: t.unsqueeze(1), (k, v))
-
-        o, l = forward(
+        o, l, should_backwards = forward(
             q, k, v,
             mask,
             attn_bias,
             attn_bias_batch_dim,
             scale,
-            causal,
-            should_backwards
+            causal
         )
 
-        ctx.should_backwards = should_backwards
-
-        if not ctx.should_backwards:
+        if not should_backwards:
             return o
 
+        ctx.should_backwards = should_backwards
+
         ctx.save_for_backward(o, l, q, k, v, mask, attn_bias)
 
         ctx.params = (
-            dtype,
             scale,
             causal,
-            attn_bias_batch_dim,
-            single_head_kv
+            attn_bias_batch_dim
         )
 
         return o
@@ -206,14 +176,10 @@ class FlashCosineSimAttention(Function):
 
         o, l, q, k, v, mask, attn_bias = ctx.saved_tensors
 
-        batch, heads, src_seq, tgt_seq, device, dtype = *q.shape[:3], k.shape[2], q.device, q.dtype
-
         (
-            dtype,
             scale,
             causal,
-            attn_bias_batch_dim,
-            single_head_kv
+            attn_bias_batch_dim
         ) = ctx.params
 
         dq, dk, dv, db = backward(
@@ -223,14 +189,10 @@ class FlashCosineSimAttention(Function):
             attn_bias,
             attn_bias_batch_dim,
             scale,
-            causal,
-            attn_bias.requires_grad
+            causal
         )
 
-        db = db if attn_bias.requires_grad else None
-
-        if single_head_kv:
-            dk, dv = map(lambda t: t.squeeze(1), (dk, dv))
+        db = db if exists(attn_bias) and attn_bias.requires_grad else None
 
         return dq, dk, dv, None, db, None, None, None, None, None, None, None, None, None, None
 
@@ -248,16 +210,10 @@ def flash_cosine_sim_attention(
     scale = 10,
     causal = False,
     l2norm_qk = True,
-    attn_bias_batch_dim = False,
-    map_qk_fn: callable = None
+    attn_bias_batch_dim = False
 ):
-    assert not (causal and exists(mask)), 'mask should not be supplied if causality is needed'
-
     if l2norm_qk:
-        q, k = l2norm_tensors(q, k)
-
-    if exists(map_qk_fn):
-        q, k = map_qk_fn(q, k)
+        q, k = l2norm_tensors(q, k)    
 
     o = flash_cosine_sim_attention_cuda(
         q, k, v,
